/******************************************************************************
    strobe.c - $strobe_compare PLI utility
    Compares pairs of signals 

Edit history
Who When      Version  What
os   4/05/02   1.2     Fixed bad malloc, synchronize problem
fy   4/23/01   1.1     Fix bad string pointer
cbs  6/19/98   1.0     Cleaned up and packaged by me
???  3/31/98   0.0     Created by someone

cbs - Chris B. Spear
os  - Oscar Strohacker
fy  - Frank Yee
******************************************************************************/

#include "vcsuser.h"
#include "acc_user.h"



/******************************************************************************
  Check the arguments to $strobe_compare
******************************************************************************/
void strobe_check() {

    if (!tf_nump()) {
	tf_message (ERR_ERROR, "Read", "ROPEN1",
		    "$strobe_compare takes at least two arguments");
	return;
    } /* if if_nump */

    if (tf_nump()%2) {
	tf_message (ERR_ERROR, "Read", "ROPEN1",
		    "$strobe_compare takes an even number of arguments, you gave %d", tf_nump());
	return;
    } /* if if_nump */

} /* strobe_check */




/******************************************************************************
  Call routine for $strobe_compare
******************************************************************************/
void strobe_call() {
  /* Call back at the end of the timeslot when everything has settled down */
  tf_rosynchronize();
  
} /* strobe_call */



/******************************************************************************
 Misc routine for $strobe_compare
******************************************************************************/
void strobe_misc(int data, int reason) {
  int        num, i, first_error;
  char       *val1, *val2;

  if (reason != REASON_ROSYNCH)
    return;

  first_error = 1;
  num = tf_nump();

  /* Step through the arguments, comparing them in pairs */
  for (i=2; i<=num; i+=2) {
    char *bits;
    /* Store the current values in temporary strings */
    bits = tf_strgetp(i-1,'b');
    val1 = strcpy((char *) malloc (strlen (bits)+1), bits);
    bits = tf_strgetp(i,'b');
    val2 = strcpy((char *) malloc (strlen (bits)+1), bits);


      /* Look for a mismatch */
      if (strcmp(val1,val2) != 0 && !acc_error_flag){ 
	  char       *exp, *found;
	  handle     obj_handle, parent_handle, eh, fh;
	  s_location s_loc;
	  p_location loc_p = &s_loc;

	  /* There is a mismatch, start error processing.
	     Errors should be rare, so only initialize when one found */
	  if (first_error) {
	      first_error = 0;
	      acc_initialize();
	      obj_handle    = acc_handle_tfinst(); 
	      parent_handle = acc_handle_parent(obj_handle);
	      acc_fetch_location(loc_p, obj_handle); 

	      /* If no file name, $strobe_compare give from command line */
	      if (!loc_p->filename) {
		  loc_p->filename = "cli";
	      }
	  }

	  exp   = acc_fetch_fullname(acc_handle_tfarg(i-1));
	  found = acc_fetch_fullname(acc_handle_tfarg(i));

	  io_printf("\"%s\", line %d: ", loc_p->filename, loc_p->line_no);
	  io_printf("Simulated value does not match expected value in %s.\n", 
		    loc_p->filename, loc_p->line_no, 
		    acc_fetch_name(parent_handle));
	  io_printf("Expected %s = %s, Simulated %s = %s at time %s \n\n",
		    exp, val1, found, val2, tf_strgettime());

/* If you can use tf_message, most of the above goes away 
	  tf_message (ERR_WARNING, "StrobeCmp", "MisComp",
		      "Expected value %s = %s does not match simulated value %s = %s\n",
		      exp, val1, found, val2);
*/

      } /* if strcmp(val1, val2) */

      free(val1);
      free(val2);
    } /* for i to num_args */

  if (!first_error) acc_close();
  
} /* strobe_misc */
