/* 
 strobe.v - Test model for $strobe compare function 
 Stimulus is applied at the rising edge of the clock and compared
 on the falling edge
 */

`timescale 1ns/1ns

module strobe;
   parameter cycle = 20;
   reg a, b, clk;
   wire sim, expect;
   
   // Generate simulated value
   and(sim, a, b);

   // Clock generator
   initial 
     begin
	clk = 0;
	forever #(cycle/2) clk = !clk;
     end
   
   
   // Compare on the falling edge of clock
   always @(negedge clk)
      $strobe_compare(expect, sim);
   
   // Generate expected value
   assign expect = a & b;
   
   // Generate stimulus
   always @(posedge clk)
       a = !a;
	
   always @(posedge a)
       b = !b;
	
   // Initialize signals, and generate an error down the road
   initial 
     begin
	$monitor ("a=%b, b=%b, sim=%b, expect=%b @%0d", a, b, sim, expect, $stime);
	a = 1'b0;
	b = 1'b0;
	#(cycle * 10)
	$display("\n*** Testbench causing mismatch - expect fireworks ***\n");
	force expect = 1;
	#(cycle * 4) $finish;
     end

endmodule /* strobe */
   


